---
id: WebDriverIO
title: WebDriverIO
---

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

## WebDriverIO

**Extends Helper**

WebDriverIO helper which wraps [webdriverio][1] library to
manipulate browser using Selenium WebDriver or PhantomJS.

WebDriverIO requires [Selenium Server and ChromeDriver/GeckoDriver to be installed][2].

### Configuration

This helper should be configured in codecept.json or codecept.conf.js

-   `url`: base url of website to be tested.
-   `browser`: browser in which to perform testing.
-   `host`:  - WebDriver host to connect.
-   `port`:  - WebDriver port to connect.
-   `protocol`:  - protocol for WebDriver server.
-   `path`:  - path to WebDriver server,
-   `restart`:  - restart browser between tests.
-   `smartWait`: (optional) **enables [SmartWait][3]**; wait for additional milliseconds for element to appear. Enable for 5 secs: "smartWait": 5000.
-   `disableScreenshots`:  - don't save screenshots on failure.
-   `fullPageScreenshots`  - make full page screenshots on failure.
-   `uniqueScreenshotNames`:  - option to prevent screenshot override if you have scenarios with the same name in different suites.
-   `keepBrowserState`:  - keep browser state between tests when `restart` is set to false.
-   `keepCookies`:  - keep cookies between tests when `restart` set to false.
-   `windowSize`: (optional) default window size. Set to `maximize` or a dimension in the format `640x480`.
-   `waitForTimeout`:  sets default wait time in _ms_ for all `wait*` functions.
-   `desiredCapabilities`: Selenium's [desired
    capabilities][4].
-   `manualStart`:  - do not start browser before a test, start it manually inside a helper
    with `this.helpers["WebDriverIO"]._startBrowser()`.
-   `timeouts`: [WebDriverIO timeouts][5] defined as hash.

Example:

```json
{
   "helpers": {
     "WebDriverIO" : {
       "smartWait": 5000,
       "browser": "chrome",
       "restart": false,
       "windowSize": "maximize",
       "timeouts": {
         "script": 60000,
         "page load": 10000
       }
     }
   }
}
```

Additional configuration params can be used from [webdriverio
website][6].

### Headless Chrome

```json
{
   "helpers": {
     "WebDriverIO" : {
       "url": "http://localhost",
       "browser": "chrome",
       "desiredCapabilities": {
         "chromeOptions": {
           "args": [ "--headless", "--disable-gpu", "--no-sandbox" ]
         }
       }
     }
   }
}
```

### Connect through proxy

CodeceptJS also provides flexible options when you want to execute tests to Selenium servers through proxy. You will
need to update the `helpers.WebDriverIO.desiredCapabilities.proxy` key.

```js
{
    "helpers": {
        "WebDriverIO": {
            "desiredCapabilities": {
                "proxy": {
                    "proxyType": "manual|pac",
                    "proxyAutoconfigUrl": "URL TO PAC FILE",
                    "httpProxy": "PROXY SERVER",
                    "sslProxy": "PROXY SERVER",
                    "ftpProxy": "PROXY SERVER",
                    "socksProxy": "PROXY SERVER",
                    "socksUsername": "USERNAME",
                    "socksPassword": "PASSWORD",
                    "noProxy": "BYPASS ADDRESSES"
                }
            }
        }
    }
}
```

For example,

```js
{
    "helpers": {
        "WebDriverIO": {
            "desiredCapabilities": {
                "proxy": {
                    "proxyType": "manual",
                    "httpProxy": "http://corporate.proxy:8080",
                    "socksUsername": "codeceptjs",
                    "socksPassword": "secret",
                    "noProxy": "127.0.0.1,localhost"
                }
            }
        }
    }
}
```

Please refer to [Selenium - Proxy Object][4] for more
information.

### Cloud Providers

WebDriverIO makes it possible to execute tests against services like `Sauce Labs` `BrowserStack` `TestingBot`
Check out their documentation on [available parameters][7]

Connecting to `BrowserStack` and `Sauce Labs` is simple. All you need to do
is set the `user` and `key` parameters. WebDriverIO automatically know which
service provider to connect to.

```js
{
    "helpers":{
        "WebDriverIO": {
            "url": "YOUR_DESIRED_HOST",
            "user": "YOUR_BROWSERSTACK_USER",
            "key": "YOUR_BROWSERSTACK_KEY",
            "desiredCapabilities": {
                "browserName": "chrome",

                // only set this if you're using BrowserStackLocal to test a local domain
                // "browserstack.local": true,

                // set this option to tell browserstack to provide addition debugging info
                // "browserstack.debug": true,
            }
        }
    }
}
```

### Multiremote Capabilities

This is a work in progress but you can control two browsers at a time right out of the box.
Individual control is something that is planned for a later version.

Here is the [webdriverio docs][8] on the subject

```js
{
    "helpers": {
        "WebDriverIO": {
            "multiremote": {
                "MyChrome": {
                    "desiredCapabilities": {
                        "browserName": "chrome"
                     }
                },
                "MyFirefox": {
                   "desiredCapabilities": {
                       "browserName": "firefox"
                   }
                }
            }
        }
    }
}
```

## Access From Helpers

Receive a WebDriverIO client from a custom helper by accessing `browser` property:

```js
this.helpers['WebDriverIO'].browser
```

### Parameters

-   `config`  

### _locate

Get elements by different locator types, including strict locator.
Should be used in custom helpers:

```js
this.helpers['WebDriverIO']._locate({name: 'password'}).then //...
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `smartWait`   

### _locateCheckable

Find a checkbox by providing human readable text:

```js
this.helpers['WebDriverIO']._locateCheckable('I agree with terms and conditions').then // ...
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.

### _locateClickable

Find a clickable element by providing human readable text:

```js
this.helpers['WebDriverIO']._locateClickable('Next page').then // ...
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.

### _locateFields

Find field elements by providing human readable text:

```js
this.helpers['WebDriverIO']._locateFields('Your email').then // ...
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.

### acceptPopup

Accepts the active JavaScript native popup window, as created by window.alert|window.confirm|window.prompt.
Don't confuse popups with modal windows, as created by [various
libraries][9]. Appium: support only web testing

### amOnPage

Opens a web page in a browser. Requires relative or absolute url.
If url starts with `/`, opens a web page of a site defined in `url` config parameter.

```js
I.amOnPage('/'); // opens main page of website
I.amOnPage('https://github.com'); // opens github
I.amOnPage('/login'); // opens a login page
```

#### Parameters

-   `url` **[string][10]** url path or global url.
    Appium: support only web testing

### appendField

Appends text to a input field or textarea.
Field is located by name, label, CSS or XPath

```js
I.appendField('#myTextField', 'appended');
```

#### Parameters

-   `field` **CodeceptJS.LocatorOrString** located by label|name|CSS|XPath|strict locator
-   `value` **[string][10]** text value to append.
    Appium: support, but it's clear a field before insert in apps

### attachFile

Attaches a file to element located by label, name, CSS or XPath
Path to file is relative current codecept directory (where codecept.json or codecept.conf.js is located).
File will be uploaded to remote system (if tests are running remotely).

```js
I.attachFile('Avatar', 'data/avatar.jpg');
I.attachFile('form input[name=avatar]', 'data/avatar.jpg');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** field located by label|name|CSS|XPath|strict locator.
-   `pathToFile` **[string][10]** local file path relative to codecept.json config file.
    Appium: not tested

### cancelPopup

Dismisses the active JavaScript popup, as created by window.alert|window.confirm|window.prompt.
Appium: support only web testing

### checkOption

Selects a checkbox or radio button.
Element is located by label or name or CSS or XPath.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
I.checkOption('#agree');
I.checkOption('I Agree to Terms and Conditions');
I.checkOption('agree', '//form');
```

#### Parameters

-   `field` **CodeceptJS.LocatorOrString** checkbox located by label | name | CSS | XPath | strict locator.
-   `context` **CodeceptJS.LocatorOrString?** (optional, `null` by default) element located by CSS | XPath | strict locator.
    Appium: not tested 

### clearCookie

{{> clearCookie}}
Appium: support only web testing

#### Parameters

-   `cookie`  

### clearField

{{> clearField}}
Appium: support

#### Parameters

-   `field`  

### click

Perform a click on a link or a button, given by a locator.
If a fuzzy locator is given, the page will be searched for a button, link, or image matching the locator string.
For buttons, the "value" attribute, "name" attribute, and inner text are searched. For links, the link text is searched.
For images, the "alt" attribute and inner text of any parent links are searched.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
// simple link
I.click('Logout');
// button of form
I.click('Submit');
// CSS button
I.click('#form input[type=submit]');
// XPath
I.click('//form/*[@type=submit]');
// link in context
I.click('Logout', '#nav');
// using strict locator
I.click({css: 'nav a.login'});
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** clickable link or button located by text, or any element located by CSS|XPath|strict locator.
-   `context` **CodeceptJS.LocatorOrString?** (optional, `null` by default) element to search in CSS|XPath|Strict locator.Appium: support 

### closeCurrentTab

Close current tab.

```js
I.closeCurrentTab();
```

### closeOtherTabs

Close all tabs except for the current one.
Appium: support web test

```js
I.closeOtherTabs();
```

### defineTimeout

Set [WebDriverIO timeouts][5] in realtime.
Appium: support only web testing.
Timeouts are expected to be passed as object:

```js
I.defineTimeout({ script: 5000 });
I.defineTimeout({ implicit: 10000, pageLoad: 10000, script: 5000 });
```

#### Parameters

-   `timeouts` **WebdriverIO.Timeouts** WebDriver timeouts object.

### dontSee

Opposite to `see`. Checks that a text is not present on a page.
Use context parameter to narrow down the search.

```js
I.dontSee('Login'); // assume we are already logged in.
I.dontSee('Login', '.nav'); // no login inside .nav element
```

#### Parameters

-   `text` **[string][10]** which is not present.
-   `context` **CodeceptJS.LocatorOrString?** (optional) element located by CSS|XPath|strict locator in which to perfrom search.Appium: support with context in apps 

### dontSeeCheckboxIsChecked

Verifies that the specified checkbox is not checked.

```js
I.dontSeeCheckboxIsChecked('#agree'); // located by ID
I.dontSeeCheckboxIsChecked('I agree to terms'); // located by label
I.dontSeeCheckboxIsChecked('agree'); // located by name
```

#### Parameters

-   `field` **CodeceptJS.LocatorOrString** located by label|name|CSS|XPath|strict locator.Appium: not tested

### dontSeeCookie

{{> dontSeeCookie}}
Appium: support only web testing

#### Parameters

-   `name`  

### dontSeeCurrentUrlEquals

Checks that current url is not equal to provided one.
If a relative url provided, a configured url will be prepended to it.

```js
I.dontSeeCurrentUrlEquals('/login'); // relative url are ok
I.dontSeeCurrentUrlEquals('http://mysite.com/login'); // absolute urls are also ok
```

#### Parameters

-   `url` **[string][10]** value to check.
    Appium: support only web testing

### dontSeeElement

{{> dontSeeElement}}
Appium: support

#### Parameters

-   `locator`  

### dontSeeElementInDOM

Opposite to `seeElementInDOM`. Checks that element is not on page.

```js
I.dontSeeElementInDOM('.nav'); // checks that element is not on page visible or not
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** located by CSS|XPath|Strict locator.
    Appium: support

### dontSeeInCurrentUrl

Checks that current url does not contain a provided fragment.

#### Parameters

-   `url` **[string][10]** value to check.
    Appium: support only web testing

### dontSeeInField

Checks that value of input field or textarea doesn't equal to given value
Opposite to `seeInField`.

```js
I.dontSeeInField('email', 'user@user.com'); // field by name
I.dontSeeInField({ css: 'form input.email' }, 'user@user.com'); // field by CSS
```

#### Parameters

-   `field` **CodeceptJS.LocatorOrString** located by label|name|CSS|XPath|strict locator.
-   `value` **[string][10]** value to check.
    Appium: support only web testing

### dontSeeInSource

Checks that the current page does not contains the given string in its raw source code.

```js
I.dontSeeInSource('<!--'); // no comments in source
```

#### Parameters

-   `text`  
-   `value` **[string][10]** to check.Appium: support

### dontSeeInTitle

Checks that title does not contain text.

```js
I.dontSeeInTitle('Error');
```

#### Parameters

-   `text` **[string][10]** value to check.
    Appium: support only web testing

### doubleClick

Performs a double-click on an element matched by link|button|label|CSS or XPath.
Context can be specified as second parameter to narrow search.

```js
I.doubleClick('Edit');
I.doubleClick('Edit', '.actions');
I.doubleClick({css: 'button.accept'});
I.doubleClick('.btn.edit');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** clickable link or button located by text, or any element located by CSS|XPath|strict locator.
-   `context` **CodeceptJS.LocatorOrString?** (optional, `null` by default) element to search in CSS|XPath|Strict locator.Appium: support only web testing 

### dragAndDrop

Drag an item to a destination element.

```js
I.dragAndDrop('#dragHandle', '#container');
```

#### Parameters

-   `srcElement` **([string][10] | [object][11])** located by CSS|XPath|strict locator.
-   `destElement` **([string][10] | [object][11])** located by CSS|XPath|strict locator.
    Appium: not tested

### executeAsyncScript

Executes async script on page.
Provided function should execute a passed callback (as first argument) to signal it is finished.

Example: In Vue.js to make components completely rendered we are waiting for [nextTick][12].

```js
I.executeAsyncScript(function(done) {
Vue.nextTick(done); // waiting for next tick
});
```

By passing value to `done()` function you can return values.
Additional arguments can be passed as well, while `done` function is always last parameter in arguments list.

```js
let val = await I.executeAsyncScript(function(url, done) {
// in browser context
$.ajax(url, { success: (data) => done(data); }
}, 'http://ajax.callback.url/');
```

#### Parameters

-   `fn` **([string][10] | [function][13])** function to be executed in browser context.
-   `args` **...any** to be passed to function.

Returns **[Promise][14]&lt;any>** Appium: support only web testing

### executeScript

Executes sync script on a page.
Pass arguments to function as additional parameters.
Will return execution result to a test.
In this case you should use async function and await to receive results.

Example with jQuery DatePicker:

```js
// change date of jQuery DatePicker
I.executeScript(function() {
// now we are inside browser context
$('date').datetimepicker('setDate', new Date());
});
```

Can return values. Don't forget to use `await` to get them.

```js
let date = await I.executeScript(function(el) {
// only basic types can be returned
return $(el).datetimepicker('getDate').toString();
}, '#date'); // passing jquery selector
```

#### Parameters

-   `fn` **([string][10] | [function][13])** function to be executed in browser context.
-   `args` **...any** to be passed to function.

Returns **[Promise][14]&lt;any>** Appium: support only web testingWraps [execute][15] command.

### fillField

Fills a text field or textarea, after clearing its value, with the given string.
Field is located by name, label, CSS, or XPath.

```js
// by label
I.fillField('Email', 'hello@world.com');
// by name
I.fillField('password', secret('123456'));
// by CSS
I.fillField('form#login input[name=username]', 'John');
// or by strict locator
I.fillField({css: 'form#login input[name=username]'}, 'John');
```

#### Parameters

-   `field` **CodeceptJS.LocatorOrString** located by label|name|CSS|XPath|strict locator.
-   `value` **[string][10]** text value to fill.Appium: support

### grabAttributeFrom

Retrieves an attribute from an element located by CSS or XPath and returns it to test.
An array as a result will be returned if there are more than one matched element.
Resumes test execution, so **should be used inside async with `await`** operator.

```js
let hint = await I.grabAttributeFrom('#tooltip', 'title');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `attr` **[string][10]** attribute name.

Returns **[Promise][14]&lt;[string][10]>** attribute value
Appium: can be used for apps only with several values ("contentDescription", "text", "className", "resourceId")

### grabBrowserLogs

Get JS log from browser. Log buffer is reset after each request.

```js
let logs = await I.grabBrowserLogs();
console.log(JSON.stringify(logs))
```

### grabCookie

{{> grabCookie}}
Appium: support only web testing

#### Parameters

-   `name`  

### grabCssPropertyFrom

Grab CSS property for given locator
Resumes test execution, so **should be used inside an async function with `await`** operator.

```js
const value = await I.grabCssPropertyFrom('h3', 'font-weight');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `cssProperty` **[string][10]** CSS property name.

Returns **[Promise][14]&lt;[string][10]>** CSS value

### grabCurrentUrl

Get current URL from browser.
Resumes test execution, so should be used inside an async function.

```js
let url = await I.grabCurrentUrl();
console.log(`Current URL is [${url}]`);
```

Returns **[Promise][14]&lt;[string][10]>** current URL

### grabHTMLFrom

Retrieves the innerHTML from an element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async function with `await`** operator.
If more than one element is found - an array of HTMLs returned.

```js
let postHTML = await I.grabHTMLFrom('#post');
```

#### Parameters

-   `locator`  
-   `element` **CodeceptJS.LocatorOrString** located by CSS|XPath|strict locator.

Returns **[Promise][14]&lt;[string][10]>** HTML code for an element
Appium: support only web testing

### grabNumberOfOpenTabs

Grab number of open tabs.

```js
let tabs = await I.grabNumberOfOpenTabs();
```

Returns **[Promise][14]&lt;[number][16]>** number of open tabs

### grabNumberOfVisibleElements

Grab number of visible elements by locator.

```js
let numOfElements = await I.grabNumberOfVisibleElements('p');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** located by CSS|XPath|strict locator.

Returns **[Promise][14]&lt;[number][16]>** number of visible elements

### grabPageScrollPosition

{{> grabPageScrollPosition}}

### grabPopupText

Grab the text within the popup. If no popup is visible then it will return null.

```js
await I.grabPopupText();
```

### grabSource

Retrieves page source and returns it to test.
Resumes test execution, so should be used inside an async function.

```js
let pageSource = await I.grabSource();
```

Returns **[Promise][14]&lt;[string][10]>** source code
Appium: support

### grabTextFrom

Retrieves a text from an element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async with `await`** operator.

```js
let pin = await I.grabTextFrom('#pin');
```

If multiple elements found returns an array of texts.

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.

Returns **[Promise][14]&lt;([string][10] | [Array][17]&lt;[string][10]>)>** attribute value
Appium: support

### grabTitle

Retrieves a page title and returns it to test.
Resumes test execution, so **should be used inside async with `await`** operator.

```js
let title = await I.grabTitle();
```

Returns **[Promise][14]&lt;[string][10]>** title
Appium: support only web testing

### grabValueFrom

Retrieves a value from a form element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async function with `await`** operator.

```js
let email = await I.grabValueFrom('input[name=email]');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** field located by label|name|CSS|XPath|strict locator.

Returns **[Promise][14]&lt;[string][10]>** attribute value
Appium: support only web testing

### moveCursorTo

{{> moveCursorTo}}
Appium: support only web testing

#### Parameters

-   `locator`  
-   `offsetX`   
-   `offsetY`   

### openNewTab

Open new tab and switch to it.

```js
I.openNewTab();
```

### pressKey

Presses a key in the browser (on a focused element).

_Hint:_ For populating text field or textarea, it is recommended to use [`fillField`][18].

```js
I.pressKey('Backspace');
```

To press a key in combination with modifier keys, pass the sequence as an array. All modifier keys (`'Alt'`, `'Control'`, `'Meta'`, `'Shift'`) will be released afterwards.

```js
I.pressKey(['Control', 'Z']);
```

For specifying operation modifier key based on operating system it is suggested to use `'CommandOrControl'`.
This will press `'Command'` (also known as `'Meta'`) on macOS machines and `'Control'` on non-macOS machines.

```js
I.pressKey(['CommandOrControl', 'Z']);
```

Some of the supported key names are:

-   `'AltLeft'` or `'Alt'`
-   `'AltRight'`
-   `'ArrowDown'`
-   `'ArrowLeft'`
-   `'ArrowRight'`
-   `'ArrowUp'`
-   `'Backspace'`
-   `'Clear'`
-   `'ControlLeft'` or `'Control'`
-   `'ControlRight'`
-   `'Command'`
-   `'CommandOrControl'`
-   `'Delete'`
-   `'End'`
-   `'Enter'`
-   `'Escape'`
-   `'F1'` to `'F12'`
-   `'Home'`
-   `'Insert'`
-   `'MetaLeft'` or `'Meta'`
-   `'MetaRight'`
-   `'Numpad0'` to `'Numpad9'`
-   `'NumpadAdd'`
-   `'NumpadDecimal'`
-   `'NumpadDivide'`
-   `'NumpadMultiply'`
-   `'NumpadSubtract'`
-   `'PageDown'`
-   `'PageUp'`
-   `'Pause'`
-   `'Return'`
-   `'ShiftLeft'` or `'Shift'`
-   `'ShiftRight'`
-   `'Space'`
-   `'Tab'`

#### Parameters

-   `key` **([string][10] | [Array][17]&lt;[string][10]>)** key or array of keys to press.

### refreshPage

Reload the current page.

```js
I.refreshPage();
```

### resizeWindow

Resize the current window to provided width and height.
First parameter can be set to `maximize`.

#### Parameters

-   `width` **[number][16]** width in pixels or `maximize`.
-   `height` **[number][16]** height in pixels.
    Appium: not tested in web, in apps doesn't work

### rightClick

Performs right click on a clickable element matched by semantic locator, CSS or XPath.

```js
// right click element with id el
I.rightClick('#el');
// right click link or button with text "Click me"
I.rightClick('Click me');
// right click button with text "Click me" inside .context
I.rightClick('Click me', '.context');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** clickable element located by CSS|XPath|strict locator.
-   `context` **CodeceptJS.LocatorOrString?** (optional, `null` by default) element located by CSS|XPath|strict locator.Appium: support, but in apps works as usual click 

### runInWeb

Placeholder for ~ locator only test case write once run on both Appium and WebDriverIO.

#### Parameters

-   `fn`  

### runOnAndroid

Placeholder for ~ locator only test case write once run on both Appium and WebDriverIO.

#### Parameters

-   `caps`  
-   `fn`  

### runOnIOS

Placeholder for ~ locator only test case write once run on both Appium and WebDriverIO.

#### Parameters

-   `caps`  
-   `fn`  

### saveScreenshot

{{> saveScreenshot}}
Appium: support

#### Parameters

-   `fileName`  
-   `fullPage`   

### scrollPageToBottom

Scroll page to the bottom.

```js
I.scrollPageToBottom();
```

### scrollPageToTop

Scroll page to the top.

```js
I.scrollPageToTop();
```

### scrollTo

Scrolls to element matched by locator.
Extra shift can be set with offsetX and offsetY options.

```js
I.scrollTo('footer');
I.scrollTo('#submit', 5, 5);
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** located by CSS|XPath|strict locator.
-   `offsetX` **[number][16]** (optional) X-axis offset. 
-   `offsetY` **[number][16]** (optional) Y-axis offset. 

### scrollTo

Scrolls to element matched by locator.
Extra shift can be set with offsetX and offsetY options.

```js
I.scrollTo('footer');
I.scrollTo('#submit', 5, 5);
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** located by CSS|XPath|strict locator.
-   `offsetX` **[number][16]** (optional, `0` by default) X-axis offset. 
-   `offsetY` **[number][16]** (optional, `0` by default) Y-axis offset.
    Appium: support only web testing 

### see

Checks that a page contains a visible text.
Use context parameter to narrow down the search.

```js
I.see('Welcome'); // text welcome on a page
I.see('Welcome', '.content'); // text inside .content div
I.see('Register', {css: 'form.register'}); // use strict locator
```

#### Parameters

-   `text` **[string][10]** expected on page.
-   `context` **CodeceptJS.LocatorOrString?** (optional, `null` by default) element located by CSS|Xpath|strict locator in which to search for text.
    Appium: support with context in apps 

### seeAttributesOnElements

Checks that all elements with given locator have given attributes.

```js
I.seeAttributesOnElements('//form', { method: "post"});
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** located by CSS|XPath|strict locator.
-   `attributes` **[object][11]** attributes and their values to check.

### seeCheckboxIsChecked

Verifies that the specified checkbox is checked.

```js
I.seeCheckboxIsChecked('Agree');
I.seeCheckboxIsChecked('#agree'); // I suppose user agreed to terms
I.seeCheckboxIsChecked({css: '#signup_form input[type=checkbox]'});
```

#### Parameters

-   `field` **CodeceptJS.LocatorOrString** located by label|name|CSS|XPath|strict locator.Appium: not tested

### seeCookie

{{> seeCookie}}
Appium: support only web testing

#### Parameters

-   `name`  

### seeCssPropertiesOnElements

Checks that all elements with given locator have given CSS properties.

```js
I.seeCssPropertiesOnElements('h3', { 'font-weight': "bold"});
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** located by CSS|XPath|strict locator.
-   `cssProperties` **[object][11]** object with CSS properties and their values to check.

### seeCurrentUrlEquals

Checks that current url is equal to provided one.
If a relative url provided, a configured url will be prepended to it.
So both examples will work:

```js
I.seeCurrentUrlEquals('/register');
I.seeCurrentUrlEquals('http://my.site.com/register');
```

#### Parameters

-   `url` **[string][10]** value to check.
    Appium: support only web testing

### seeElement

Checks that a given Element is visible
Element is located by CSS or XPath.

```js
I.seeElement('#modal');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** located by CSS|XPath|strict locator.
    Appium: support

### seeElementInDOM

Checks that a given Element is present in the DOM
Element is located by CSS or XPath.

```js
I.seeElementInDOM('#modal');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.Appium: support

### seeInCurrentUrl

Checks that current url contains a provided fragment.

```js
I.seeInCurrentUrl('/register'); // we are on registration page
```

#### Parameters

-   `url` **[string][10]** a fragment to check
    Appium: support only web testing

### seeInField

Checks that the given input field or textarea equals to given value.
For fuzzy locators, fields are matched by label text, the "name" attribute, CSS, and XPath.

```js
I.seeInField('Username', 'davert');
I.seeInField({css: 'form textarea'},'Type your comment here');
I.seeInField('form input[type=hidden]','hidden_value');
I.seeInField('#searchform input','Search');
```

#### Parameters

-   `field` **CodeceptJS.LocatorOrString** located by label|name|CSS|XPath|strict locator.
-   `value` **[string][10]** value to check.Appium: support only web testing

### seeInPopup

Checks that the active JavaScript popup, as created by `window.alert|window.confirm|window.prompt`, contains the
given string. Appium: support only web testing

#### Parameters

-   `text` **[string][10]** value to check.

### seeInSource

Checks that the current page contains the given string in its raw source code.

```js
I.seeInSource('<h1>Green eggs &amp; ham</h1>');
```

#### Parameters

-   `text` **[string][10]** value to check.
    Appium: support

### seeInTitle

Checks that title contains text.

```js
I.seeInTitle('Home Page');
```

#### Parameters

-   `text` **[string][10]** text value to check.
    Appium: support only web testing

### seeNumberOfElements

Asserts that an element appears a given number of times in the DOM.
Element is located by label or name or CSS or XPath.
Appium: support

```js
I.seeNumberOfElements('#submitBtn', 1);
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `num` **[number][16]?** number of elements.

### seeNumberOfVisibleElements

Asserts that an element is visible a given number of times.
Element is located by CSS or XPath.

```js
I.seeNumberOfVisibleElements('.buttons', 3);
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `num` **[number][16]** number of elements.

### seeTextEquals

Checks that text is equal to provided one.

```js
I.seeTextEquals('text', 'h1');
```

#### Parameters

-   `text` **[string][10]** element value to check.
-   `context` **CodeceptJS.LocatorOrString??** (optional) element located by CSS|XPath|strict locator. 

### seeTitleEquals

Checks that title is equal to provided one.

```js
I.seeTitleEquals('Test title.');
```

#### Parameters

-   `text` **[string][10]** value to check.

### selectOption

{{> selectOption}}

#### Parameters

-   `select`  
-   `option`  

### setCookie

{{> setCookie}}
Appium: support only web testing

Uses Selenium's JSON [cookie
format][19].

#### Parameters

-   `cookie`  

### switchTo

Switches frame or in case of null locator reverts to parent.

```js
I.switchTo('iframe'); // switch to first iframe
I.switchTo(); // switch back to main page
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString?** (optional, `null` by default) element located by CSS|XPath|strict locator.
    Appium: support only web testing 

### switchToNextTab

Switch focus to a particular tab by its number. It waits tabs loading and then switch tab.

```js
I.switchToNextTab();
I.switchToNextTab(2);
```

#### Parameters

-   `num` **[number][16]** (optional) number of tabs to switch forward, default: 1. 
-   `sec` **[number][16]?** (optional) time in seconds to wait. 

### switchToPreviousTab

Switch focus to a particular tab by its number. It waits tabs loading and then switch tab.

```js
I.switchToPreviousTab();
I.switchToPreviousTab(2);
```

#### Parameters

-   `num` **[number][16]** (optional) number of tabs to switch backward, default: 1. 
-   `sec` **[number][16]??** (optional) time in seconds to wait. 

### uncheckOption

Unselects a checkbox or radio button.
Element is located by label or name or CSS or XPath.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
I.uncheckOption('#agree');
I.uncheckOption('I Agree to Terms and Conditions');
I.uncheckOption('agree', '//form');
```

#### Parameters

-   `field` **CodeceptJS.LocatorOrString** checkbox located by label | name | CSS | XPath | strict locator.
-   `context` **CodeceptJS.LocatorOrString?** (optional, `null` by default) element located by CSS | XPath | strict locator.
    Appium: not tested 

### wait

Pauses execution for a number of seconds.

```js
I.wait(2); // wait 2 secs
```

#### Parameters

-   `sec` **[number][16]** number of second to wait.
    Appium: support

### waitForDetached

Waits for an element to become not attached to the DOM on a page (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitForDetached('#popup');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait
    Appium: support 

### waitForElement

Waits for element to be present on page (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitForElement('.btn.continue');
I.waitForElement('.btn.continue', 5); // wait for 5 secs
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `sec` **[number][16]?** (optional, `1` by default) time in seconds to wait
    Appium: support 

### waitForEnabled

Waits for element to become enabled (by default waits for 1sec).
Element can be located by CSS or XPath.

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `sec` **[number][16]** (optional) time in seconds to wait, 1 by default.
    Appium: support 

### waitForFunction

Waits for a function to return true (waits for 1 sec by default).
Running in browser context.

```js
I.waitForFunction(fn[, [args[, timeout]])
```

```js
I.waitForFunction(() => window.requests == 0);
I.waitForFunction(() => window.requests == 0, 5); // waits for 5 sec
I.waitForFunction((count) => window.requests == count, [3], 5) // pass args and wait for 5 sec
```

#### Parameters

-   `fn` **([string][10] | [function][13])** to be executed in browser context.
-   `argsOrSec` **([Array][17]&lt;any> | [number][16])?** (optional, `1` by default) arguments for function or seconds. 
-   `sec` **[number][16]?** (optional, `1` by default) time in seconds to waitAppium: support 

### waitForInvisible

Waits for an element to be removed or become invisible on a page (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitForInvisible('#popup');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait
    Appium: support 

### waitForText

Waits for a text to appear (by default waits for 1sec).
Element can be located by CSS or XPath.
Narrow down search results by providing context.

```js
I.waitForText('Thank you, form has been submitted');
I.waitForText('Thank you, form has been submitted', 5, '#modal');
```

#### Parameters

-   `text` **[string][10]** to wait for.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait 
-   `context` **CodeceptJS.LocatorOrString?** (optional) element located by CSS|XPath|strict locator.
    Appium: support 

### waitForValue

Waits for the specified value to be in value attribute.

```js
I.waitForValue('//input', "GoodValue");
```

#### Parameters

-   `field` **([string][10] | [object][11])** input field.
-   `value` **[string][10]** expected value.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait 

### waitForVisible

Waits for an element to become visible on a page (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitForVisible('#popup');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait
    Appium: support 

### waitInUrl

Waiting for the part of the URL to match the expected. Useful for SPA to understand that page was changed.

```js
I.waitInUrl('/info', 2);
```

#### Parameters

-   `urlPart` **[string][10]** value to check.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait 

### waitNumberOfVisibleElements

Waits for a specified number of elements on the page.

```js
I.waitNumberOfVisibleElements('a', 3);
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `num` **[number][16]** number of elements.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait 

### waitToHide

Waits for an element to hide (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitToHide('#popup');
```

#### Parameters

-   `locator` **CodeceptJS.LocatorOrString** element located by CSS|XPath|strict locator.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait
    Appium: support 

### waitUntil

Waits for a function to return true (waits for 1sec by default).

```js
I.waitUntil(() => window.requests == 0);
I.waitUntil(() => window.requests == 0, 5);
```

#### Parameters

-   `fn` **([function][13] | [string][10])** function which is executed in browser context.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait 
-   `timeoutMsg` **[string][10]** message to show in case of timeout fail. 
-   `interval` **[number][16]?** -   _Appium_: supported 

### waitUrlEquals

Waits for the entire URL to match the expected

```js
I.waitUrlEquals('/info', 2);
I.waitUrlEquals('http://127.0.0.1:8000/info');
```

#### Parameters

-   `urlPart` **[string][10]** value to check.
-   `sec` **[number][16]** (optional, `1` by default) time in seconds to wait 

[1]: http://webdriver.io/

[2]: http://codecept.io/quickstart/#prepare-selenium-server

[3]: http://codecept.io/acceptance/#smartwait

[4]: https://github.com/SeleniumHQ/selenium/wiki/DesiredCapabilities

[5]: http://webdriver.io/guide/testrunner/timeouts.html

[6]: http://webdriver.io/guide/getstarted/configuration.html

[7]: http://webdriver.io/guide/usage/cloudservices.html

[8]: http://webdriver.io/guide/usage/multiremote.html

[9]: http://jster.net/category/windows-modals-popups

[10]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String

[11]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object

[12]: https://vuejs.org/v2/api/#Vue-nextTick

[13]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function

[14]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Promise

[15]: http://webdriver.io/api/protocol/execute.html

[16]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number

[17]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array

[18]: #fillfield

[19]: https://code.google.com/p/selenium/wiki/JsonWireProtocol#Cookie_JSON_Object
